@@ -1,6 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Shell = imports.gi.Shell;
 const Lang = imports.lang;
@@ -22,6 +23,7 @@ const Workspace = imports.ui.workspace;
 const Params = imports.misc.params;
 
 const MENU_POPUP_TIMEOUT = 600;
+const SCROLL_TIME = 0.1;
 
 function AlphabeticalView() {
     this._init();
@@ -66,6 +68,7 @@ AlphabeticalView.prototype = {
         let appIcon = new AppWellIcon(this._appSystem.get_app(appInfo.get_id()));
 
         this._grid.addItem(appIcon.actor);
+        appIcon.actor.connect('key-focus-in', Lang.bind(this, this._ensureIconVisible));
 
         appIcon._appInfo = appInfo;
         if (this._filterApp && !this._filterApp(appInfo))
@@ -74,6 +77,28 @@ AlphabeticalView.prototype = {
         this._apps.push(appIcon);
     },
 
+    _ensureIconVisible: function(icon) {
+        let adjustment = this.actor.vscroll.adjustment;
+        let [value, lower, upper, stepIncrement, pageIncrement, pageSize] = adjustment.get_values();
+
+        let offset = 0;
+        let vfade = this.actor.get_effect("vfade");
+        if (vfade)
+            offset = vfade.fade_offset;
+
+        if (icon.y < value + offset)
+            value = Math.max(0, icon.y - offset);
+        else if (icon.y + icon.height > value + pageSize - offset)
+            value = Math.min(upper, icon.y + icon.height + offset - pageSize);
+        else
+            return;
+
+        Tweener.addTween(adjustment,
+                         { value: value,
+                           time: SCROLL_TIME,
+                           transition: 'easeOutQuad' });
+    },
+
     setFilter: function(filter) {
         this._filterApp = filter;
         for (let i = 0; i < this._apps.length; i++)
@@ -108,7 +133,14 @@ ViewByCategories.prototype = {
 
         this._view = new AlphabeticalView();
 
-        this._filters = new St.BoxLayout({ vertical: true });
+        // categories can be -1 (the All view) or 0...n-1, where n
+        // is the number of sections
+        // -2 is a flag to indicate that nothing is selected
+        // (used only before the actor is mapped the first time)
+        this._currentCategory = -2;
+        this._filters = new St.BoxLayout({ vertical: true, reactive: true });
+        this._filters.connect('scroll-event', Lang.bind(this, this._scrollFilter));
+
         this.actor.add(this._view.actor, { expand: true, x_fill: true, y_fill: true });
         this.actor.add(this._filters, { expand: false, y_fill: false, y_align: St.Align.START });
 
@@ -120,9 +152,28 @@ ViewByCategories.prototype = {
             }));
 
         this._sections = [];
+
+        // We need a dummy actor to catch the keyboard focus if the
+        // user Ctrl-Alt-Tabs here before the deferred work creates
+        // our real contents
+        this._focusDummy = new St.Bin({ can_focus: true });
+        this.actor.add(this._focusDummy);
+    },
+
+    _scrollFilter: function(actor, event) {
+        let direction = event.get_scroll_direction();
+        if (direction == Clutter.ScrollDirection.UP)
+            this._selectCategory(Math.max(this._currentCategory - 1, -1))
+        else if (direction == Clutter.ScrollDirection.DOWN)
+            this._selectCategory(Math.min(this._currentCategory + 1, this._sections.length - 1));
     },
 
     _selectCategory: function(num) {
+        if (this._currentCategory == num) // nothing to do
+            return;
+
+        this._currentCategory = num;
+
         if (num != -1)
             this._allFilter.remove_style_pseudo_class('selected');
         else
@@ -143,9 +194,10 @@ ViewByCategories.prototype = {
     },
 
     _addFilter: function(name, num) {
-        let button = new St.Button({ label: name,
+        let button = new St.Button({ label: GLib.markup_escape_text (name, -1),
                                      style_class: 'app-filter',
-                                     x_align: St.Align.START });
+                                     x_align: St.Align.START,
+                                     can_focus: true });
         this._filters.add(button, { expand: true, x_fill: true, y_fill: false });
         button.connect('clicked', Lang.bind(this, function() {
             this._selectCategory(num);
@@ -180,6 +232,14 @@ ViewByCategories.prototype = {
             this._addFilter(sections[i], i);
 
         this._selectCategory(-1);
+
+        if (this._focusDummy) {
+            let focused = this._focusDummy.has_key_focus();
+            this._focusDummy.destroy();
+            this._focusDummy = null;
+            if (focused)
+                this.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+        }
     }
 };
 
@@ -230,7 +290,10 @@ BaseAppSearchProvider.prototype = {
             return null;
         return { 'id': resultId,
                  'name': app.get_name(),
-                 'icon': app.create_icon_texture(Search.RESULT_ICON_SIZE)};
+                 'createIcon': function(size) {
+                                   return app.create_icon_texture(size);
+                               }
+               };
     },
 
     activateResult: function(id, params) {
@@ -273,10 +336,6 @@ AppSearchProvider.prototype = {
         let app = this._appSys.get_app(resultMeta['id']);
         let icon = new AppWellIcon(app);
         return icon.actor;
-    },
-
-    expandSearch: function(terms) {
-        log('TODO expand search');
     }
 };
 
@@ -288,7 +347,7 @@ PrefsSearchProvider.prototype = {
     __proto__: BaseAppSearchProvider.prototype,
 
     _init: function() {
-        BaseAppSearchProvider.prototype._init.call(this, _("PREFERENCES"));
+        BaseAppSearchProvider.prototype._init.call(this, _("SETTINGS"));
     },
 
     getInitialResultSet: function(terms) {
@@ -297,12 +356,6 @@ PrefsSearchProvider.prototype = {
 
     getSubsearchResultSet: function(previousResults, terms) {
         return this._appSys.subsearch(true, previousResults, terms);
-    },
-
-    expandSearch: function(terms) {
-        let controlCenter = this._appSys.load_from_desktop_file('gnomecc.desktop');
-        controlCenter.launch();
-        Main.overview.hide();
     }
 };
 
@@ -338,6 +391,7 @@ AppWellIcon.prototype = {
         this.actor = new St.Button({ style_class: 'app-well-app',
                                      reactive: true,
                                      button_mask: St.ButtonMask.ONE | St.ButtonMask.TWO,
+                                     can_focus: true,
                                      x_fill: true,
                                      y_fill: true });
         this.actor._delegate = this;
@@ -347,6 +401,7 @@ AppWellIcon.prototype = {
 
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
+        this.actor.connect('popup-menu', Lang.bind(this, this._onKeyboardPopupMenu));
 
         this._menu = null;
         this._menuManager = new PopupMenu.PopupMenuManager(this);
@@ -423,6 +478,11 @@ AppWellIcon.prototype = {
         return false;
     },
 
+    _onKeyboardPopupMenu: function() {
+        this.popupMenu();
+        this._menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+    },
+
     getId: function() {
         return this.app.get_id();
     },
@@ -440,6 +500,7 @@ AppWellIcon.prototype = {
                 if (!isPoppedUp)
                     this._onMenuPoppedDown();
             }));
+            Main.overview.connect('hiding', Lang.bind(this, function () { this._menu.close(); }));
 
             this._menuManager.addMenu(this._menu);
         }
@@ -548,10 +609,9 @@ AppIconMenu.prototype = {
 
         let isFavorite = AppFavorites.getAppFavorites().isFavorite(this._source.app.get_id());
 
-        this._newWindowMenuItem = windows.length > 0 ? this._appendMenuItem(_("New Window")) : null;
+        this._newWindowMenuItem = this._appendMenuItem(_("New Window"));
+        this._appendSeparator();
 
-        if (windows.length > 0)
-            this._appendSeparator();
         this._toggleFavoriteMenuItem = this._appendMenuItem(isFavorite ? _("Remove from Favorites")
                                                                     : _("Add to Favorites"));
 
