@@ -48,6 +48,7 @@ DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
 
 let chrome = null;
 let panel = null;
+let hotCorners = [];
 let placesManager = null;
 let overview = null;
 let runDialog = null;
@@ -87,7 +88,6 @@ function start() {
 
     Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
-    global.grab_dbus_service();
     shellDBusService = new ShellDBus.GnomeShell();
     // Force a connection now; dbus.js will do this internally
     // if we use its name acquisition stuff but we aren't right
@@ -95,6 +95,10 @@ function start() {
     // back into sync ones.
     DBus.session.flush();
 
+    // Load the calendar server. Note that we are careful about
+    // not loading any events until the user presses the clock
+    global.launch_calendar_server();
+
     Environment.init();
 
     // Ensure ShellWindowTracker and ShellAppUsage are initialized; this will
@@ -135,6 +139,7 @@ function start() {
 
     placesManager = new PlaceDisplay.PlacesManager();
     xdndHandler = new XdndHandler.XdndHandler();
+    ctrlAltTabManager = new CtrlAltTab.CtrlAltTabManager();
     overview = new Overview.Overview();
     chrome = new Chrome.Chrome();
     magnifier = new Magnifier.Magnifier();
@@ -149,9 +154,6 @@ function start() {
     overview.init();
     statusIconDispatcher.start(messageTray.actor);
 
-    ctrlAltTabManager = new CtrlAltTab.CtrlAltTabManager();
-    ctrlAltTabManager.addGroup(panel.actor, _("Panel"), 'gnome-panel');
-
     _startDate = new Date();
 
     let recorderSettings = new Gio.Settings({ schema: 'org.gnome.shell.recorder' });
@@ -192,6 +194,9 @@ function start() {
     ExtensionSystem.init();
     ExtensionSystem.loadExtensions();
 
+    // Perform initial relayout here
+    _relayout();
+
     panel.startStatusArea();
     panel.startupAnimation();
 
@@ -200,9 +205,6 @@ function start() {
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
-    // Perform initial relayout here
-    _relayout();
-
     _log('info', 'loaded at ' + _startDate);
     log('GNOME Shell started at ' + _startDate);
 
@@ -220,12 +222,28 @@ function start() {
 let _workspaces = [];
 let _checkWorkspacesId = 0;
 
+/*
+ * When the last window closed on a workspace is a dialog or splash
+ * screen, we assume that it might be an initial window shown before
+ * the main window of an application, and give the app a grace period
+ * where it can map another window before we remove the workspace.
+ */
+const LAST_WINDOW_GRACE_TIME = 1000;
+
 function _checkWorkspaces() {
     let i;
     let emptyWorkspaces = [];
 
-    for (i = 0; i < _workspaces.length; i++)
-        emptyWorkspaces[i] = true;
+    for (i = 0; i < _workspaces.length; i++) {
+        let lastRemoved = _workspaces[i]._lastRemovedWindow;
+        if (lastRemoved &&
+            (lastRemoved.get_window_type() == Meta.WindowType.SPLASHSCREEN ||
+             lastRemoved.get_window_type() == Meta.WindowType.DIALOG ||
+             lastRemoved.get_window_type() == Meta.WindowType.MODAL_DIALOG))
+                emptyWorkspaces[i] = false;
+        else
+            emptyWorkspaces[i] = true;
+    }
 
     let windows = global.get_window_actors();
     for (i = 0; i < windows.length; i++) {
@@ -254,6 +272,17 @@ function _checkWorkspaces() {
     return false;
 }
 
+function _windowRemoved(workspace, window) {
+    workspace._lastRemovedWindow = window;
+    _queueCheckWorkspaces();
+    Mainloop.timeout_add(LAST_WINDOW_GRACE_TIME, function() {
+        if (workspace._lastRemovedWindow == window) {
+            workspace._lastRemovedWindow = null;
+            _queueCheckWorkspaces();
+        }
+    });
+}
+
 function _queueCheckWorkspaces() {
     if (_checkWorkspacesId == 0)
         _checkWorkspacesId = Meta.later_add(Meta.LaterType.BEFORE_REDRAW, _checkWorkspaces);
@@ -277,7 +306,7 @@ function _nWorkspacesChanged() {
         for (w = oldNumWorkspaces; w < newNumWorkspaces; w++) {
             let workspace = _workspaces[w];
             workspace._windowAddedId = workspace.connect('window-added', _queueCheckWorkspaces);
-            workspace._windowRemovedId = workspace.connect('window-removed', _queueCheckWorkspaces);
+            workspace._windowRemovedId = workspace.connect('window-removed', _windowRemoved);
         }
 
     } else {
@@ -388,9 +417,34 @@ function _getAndClearErrorStack() {
 }
 
 function _relayout() {
+    let monitors = global.get_monitors();
+    if (monitors.length != hotCorners.length) {
+        // destroy old corners
+        for (let i = 0; i < hotCorners.length; i++)
+            hotCorners[i].destroy();
+        hotCorners = [];
+        for (let i = 0; i < monitors.length; i++)
+            hotCorners[i] = new Panel.HotCorner();
+    }
+
+
     let primary = global.get_primary_monitor();
-    panel.actor.set_position(primary.x, primary.y);
-    panel.actor.set_size(primary.width, Panel.PANEL_HEIGHT);
+    for (let i = 0; i < monitors.length; i++) {
+        let monitor = monitors[i];
+        let corner = hotCorners[i];
+        let isPrimary = (monitor.x == primary.x &&
+                         monitor.y == primary.y &&
+                         monitor.width == primary.width &&
+                         monitor.height == primary.height);
+        if (St.Widget.get_default_direction() == St.TextDirection.RTL)
+            corner.actor.set_position(monitor.x + monitor.width, monitor.y);
+        else
+            corner.actor.set_position(monitor.x, monitor.y);
+        if (isPrimary)
+            panel.setHotCorner(corner);
+    }
+
+    panel.relayout();
     overview.relayout();
 
     // To avoid updating the position and size of the workspaces
@@ -483,10 +537,8 @@ function _globalKeyPressHandler(actor, event) {
 
 function _findModal(actor) {
     for (let i = 0; i < modalActorFocusStack.length; i++) {
-        let [stackActor, stackFocus] = modalActorFocusStack[i];
-        if (stackActor == actor) {
+        if (modalActorFocusStack[i].actor == actor)
             return i;
-        }
     }
     return -1;
 }
@@ -512,7 +564,6 @@ function _findModal(actor) {
  * Returns: true iff we successfully acquired a grab or already had one
  */
 function pushModal(actor, timestamp) {
-
     if (timestamp == undefined)
         timestamp = global.get_current_time();
 
@@ -526,20 +577,24 @@ function pushModal(actor, timestamp) {
     global.set_stage_input_mode(Shell.StageInputMode.FULLSCREEN);
 
     modalCount += 1;
-    actor.connect('destroy', function() {
+    let actorDestroyId = actor.connect('destroy', function() {
         let index = _findModal(actor);
         if (index >= 0)
             modalActorFocusStack.splice(index, 1);
     });
     let curFocus = global.stage.get_key_focus();
+    let curFocusDestroyId;
     if (curFocus != null) {
-        curFocus.connect('destroy', function() {
+        curFocusDestroyId = curFocus.connect('destroy', function() {
             let index = _findModal(actor);
             if (index >= 0)
-                modalActorFocusStack[index][1] = null;
+                modalActorFocusStack[index].actor = null;
         });
     }
-    modalActorFocusStack.push([actor, curFocus]);
+    modalActorFocusStack.push({ actor: actor,
+                                focus: curFocus,
+                                destroyId: actorDestroyId,
+                                focusDestroyId: curFocusDestroyId });
 
     global.stage.set_key_focus(actor);
     return true;
@@ -559,28 +614,42 @@ function pushModal(actor, timestamp) {
  * global.get_current_time() is assumed.
  */
 function popModal(actor, timestamp) {
-
     if (timestamp == undefined)
         timestamp = global.get_current_time();
 
-    modalCount -= 1;
     let focusIndex = _findModal(actor);
-    if (focusIndex >= 0) {
-        if (focusIndex == modalActorFocusStack.length - 1) {
-            let [stackActor, stackFocus] = modalActorFocusStack[focusIndex];
-            global.stage.set_key_focus(stackFocus);
-        } else {
-            // Remove from the middle, shift the focus chain up
-            for (let i = focusIndex; i < modalActorFocusStack.length - 1; i++) {
-                modalActorFocusStack[i + 1][1] = modalActorFocusStack[i][1];
-            }
-        }
-        modalActorFocusStack.splice(focusIndex, 1);
+    if (focusIndex < 0) {
+        global.stage.set_key_focus(null);
+        global.end_modal(timestamp);
+        global.set_stage_input_mode(Shell.StageInputMode.NORMAL);
+
+        throw new Error('incorrect pop');
     }
+
+    modalCount -= 1;
+
+    let record = modalActorFocusStack[focusIndex];
+    record.actor.disconnect(record.destroyId);
+
+    if (focusIndex == modalActorFocusStack.length - 1) {
+        if (record.focus)
+            record.focus.disconnect(record.focusDestroyId);
+        global.stage.set_key_focus(record.focus);
+    } else {
+        let t = modalActorFocusStack[modalActorFocusStack.length - 1];
+        if (t.focus)
+            t.focus.disconnect(t.focusDestroyId);
+        // Remove from the middle, shift the focus chain up
+        for (let i = modalActorFocusStack.length - 1; i > focusIndex; i--) {
+            modalActorFocusStack[i].focus = modalActorFocusStack[i - 1].focus;
+            modalActorFocusStack[i].focusDestroyId = modalActorFocusStack[i - 1].focusDestroyId;
+        }
+    }
+    modalActorFocusStack.splice(focusIndex, 1);
+
     if (modalCount > 0)
         return;
 
-    global.stage.set_key_focus(null);
     global.end_modal(timestamp);
     global.set_stage_input_mode(Shell.StageInputMode.NORMAL);
 }
