@@ -180,10 +180,10 @@ Overview.prototype = {
         this._group.add_actor(this.viewSelector.actor);
 
         this._workspacesDisplay = new WorkspacesView.WorkspacesDisplay();
-        this.viewSelector.addViewTab(_("Windows"), this._workspacesDisplay.actor);
+        this.viewSelector.addViewTab(_("Windows"), this._workspacesDisplay.actor, 'text-x-generic');
 
         let appView = new AppDisplay.AllAppDisplay();
-        this.viewSelector.addViewTab(_("Applications"), appView.actor);
+        this.viewSelector.addViewTab(_("Applications"), appView.actor, 'system-run');
 
         // Default search providers
         this.viewSelector.addSearchProvider(new AppDisplay.AppSearchProvider());
@@ -197,6 +197,10 @@ Overview.prototype = {
         this.dash.actor.add_constraint(this.viewSelector.constrainY);
         this.dash.actor.add_constraint(this.viewSelector.constrainHeight);
 
+        // Translators: this is the name of the dock/favorites area on
+        // the left of the overview
+        Main.ctrlAltTabManager.addGroup(this.dash.actor, _("Dash"), 'user-bookmarks');
+
     },
 
     _onDragBegin: function() {
@@ -213,10 +217,19 @@ Overview.prototype = {
             global.screen.get_workspace_by_index(this._lastActiveWorkspaceIndex).activate(time);
             this.hideTemporarily();
         }
+        this._resetWindowSwitchTimeout();
         this._lastHoveredWindow = null;
         DND.removeMonitor(this._dragMonitor);
     },
 
+    _resetWindowSwitchTimeout: function() {
+        if (this._windowSwitchTimeoutId != 0) {
+            Mainloop.source_remove(this._windowSwitchTimeoutId);
+            this._windowSwitchTimeoutId = 0;
+            this._needsFakePointerEvent = false;
+        }
+    },
+
     _fakePointerEvent: function() {
         let display = Gdk.Display.get_default();
         let deviceManager = display.get_device_manager();
@@ -231,21 +244,18 @@ Overview.prototype = {
                              dragEvent.targetActor._delegate &&
                              dragEvent.targetActor._delegate.metaWindow;
 
+        this._windowSwitchTimestamp = global.get_current_time();
+
         if (targetIsWindow &&
             dragEvent.targetActor._delegate.metaWindow == this._lastHoveredWindow)
             return DND.DragMotionResult.CONTINUE;
 
         this._lastHoveredWindow = null;
 
-        if (this._windowSwitchTimeoutId != 0) {
-            Mainloop.source_remove(this._windowSwitchTimeoutId);
-            this._windowSwitchTimeoutId = 0;
-            this._needsFakePointerEvent = false;
-        }
+        this._resetWindowSwitchTimeout();
 
         if (targetIsWindow) {
             this._lastHoveredWindow = dragEvent.targetActor._delegate.metaWindow;
-            this._windowSwitchTimestamp = global.get_current_time();
             this._windowSwitchTimeoutId = Mainloop.timeout_add(DND_WINDOW_SWITCH_TIMEOUT,
                                             Lang.bind(this, function() {
                                                 this._needsFakePointerEvent = true;
@@ -507,7 +517,7 @@ Overview.prototype = {
         if (this._shown)
             return;
         // Do this manually instead of using _syncInputMode, to handle failure
-        if (!Main.pushModal(this.viewSelector.actor))
+        if (!Main.pushModal(this._group))
             return;
         this._modal = true;
         this._animateVisible();
@@ -638,18 +648,6 @@ Overview.prototype = {
             this.show();
     },
 
-    /**
-     * getWorkspacesForWindow:
-     * @metaWindow: A #MetaWindow
-     *
-     * Returns the Workspaces object associated with the given window.
-     * This method is not be accessible if the overview is not open
-     * and will return %null.
-     */
-    getWorkspacesForWindow: function(metaWindow) {
-        return this.workspaces;
-    },
-
     //// Private methods ////
 
     _syncInputMode: function() {
@@ -661,20 +659,20 @@ Overview.prototype = {
 
         if (this._shown) {
             if (!this._modal) {
-                if (Main.pushModal(this.dash.actor))
+                if (Main.pushModal(this._group))
                     this._modal = true;
                 else
                     this.hide();
             }
         } else if (this._shownTemporarily) {
             if (this._modal) {
-                Main.popModal(this.dash.actor);
+                Main.popModal(this._group);
                 this._modal = false;
             }
             global.stage_input_mode = Shell.StageInputMode.FULLSCREEN;
         } else {
             if (this._modal) {
-                Main.popModal(this.dash.actor);
+                Main.popModal(this._group);
                 this._modal = false;
             }
             else if (global.stage_input_mode == Shell.StageInputMode.FULLSCREEN)
