@@ -13,6 +13,7 @@ const Gettext = imports.gettext.domain('gnome-shell');
 const _ = Gettext.gettext;
 
 const Config = imports.misc.config;
+const CtrlAltTab = imports.ui.ctrlAltTab;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
@@ -25,6 +26,8 @@ const PANEL_HEIGHT = 26;
 
 const PANEL_ICON_SIZE = 24;
 
+const STARTUP_ANIMATION_TIME = 0.2;
+
 const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;
 
 const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
@@ -44,14 +47,6 @@ const STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION = {
 if (Config.HAVE_BLUETOOTH)
     STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION['bluetooth'] = imports.ui.status.bluetooth.Indicator;
 
-// in org.gnome.desktop.interface
-const CLOCK_FORMAT_KEY        = 'clock-format';
-
-// in org.gnome.shell.clock
-const CLOCK_SHOW_DATE_KEY     = 'show-date';
-const CLOCK_SHOW_SECONDS_KEY  = 'show-seconds';
-
-
 // To make sure the panel corners blend nicely with the panel,
 // we draw background and borders the same way, e.g. drawing
 // them as filled shapes from the outside inwards instead of
@@ -281,7 +276,7 @@ AppMenuButton.prototype = {
         this._clipWidth = PANEL_ICON_SIZE;
         this._direction = SPINNER_SPEED;
 
-        this._spinner = new AnimatedIcon('process-working.png',
+        this._spinner = new AnimatedIcon('process-working.svg',
                                          PANEL_ICON_SIZE);
         this._container.add_actor(this._spinner.actor);
         this._spinner.actor.lower_bottom();
@@ -410,7 +405,7 @@ AppMenuButton.prototype = {
         let [minSize, naturalSize] = this._iconBox.get_preferred_height(forWidth);
         alloc.min_size = minSize;
         alloc.natural_size = naturalSize;
-        [minSizfe, naturalSize] = this._label.actor.get_preferred_height(forWidth);
+        [minSize, naturalSize] = this._label.actor.get_preferred_height(forWidth);
         if (minSize > alloc.min_size)
             alloc.min_size = minSize;
         if (naturalSize > alloc.natural_size)
@@ -560,7 +555,7 @@ PanelCorner.prototype = {
         this._side = side;
         this.actor = new St.DrawingArea({ style_class: 'panel-corner' });
         this.actor.connect('repaint', Lang.bind(this, this._repaint));
-        this.actor.connect('style-changed', Lang.bind(this, this._reposition));
+        this.actor.connect('style-changed', Lang.bind(this, this.relayout));
     },
 
     _repaint: function() {
@@ -580,18 +575,19 @@ PanelCorner.prototype = {
         cr.moveTo(0, 0);
         if (this._side == St.Side.LEFT)
             cr.arc(cornerRadius,
-                   innerBorderWidth + outerBorderWidth + cornerRadius,
+                   innerBorderWidth + cornerRadius,
                    cornerRadius, Math.PI, 3 * Math.PI / 2);
         else
             cr.arc(0,
-                   innerBorderWidth + outerBorderWidth + cornerRadius,
+                   innerBorderWidth + cornerRadius,
                    cornerRadius, 3 * Math.PI / 2, 2 * Math.PI);
         cr.lineTo(cornerRadius, 0);
         cr.closePath();
 
         let savedPath = cr.copyPath();
 
-        let over = _over(outerBorderColor, backgroundColor);
+        let over = _over(innerBorderColor,
+                         _over(outerBorderColor, backgroundColor));
         Clutter.cairo_set_source_color(cr, over);
         cr.fill();
 
@@ -607,12 +603,12 @@ PanelCorner.prototype = {
         cr.restore();
 
         if (this._side == St.Side.LEFT)
-            cr.rectangle(cornerRadius - offset, 0, offset, innerBorderWidth);
+            cr.rectangle(cornerRadius - offset, 0, offset, outerBorderWidth);
         else
-            cr.rectangle(0, 0, offset, innerBorderWidth);
+            cr.rectangle(0, 0, offset, outerBorderWidth);
         cr.fill();
 
-        offset = innerBorderWidth + outerBorderWidth;
+        offset = innerBorderWidth;
         Clutter.cairo_set_source_color(cr, backgroundColor);
 
         cr.save();
@@ -622,24 +618,177 @@ PanelCorner.prototype = {
         cr.restore();
     },
 
-    _reposition: function() {
+    relayout: function() {
         let node = this.actor.get_theme_node();
 
         let cornerRadius = node.get_length("-panel-corner-radius");
         let innerBorderWidth = node.get_length('-panel-corner-inner-border-width');
-        let outerBorderWidth = node.get_length('-panel-corner-outer-border-width');
 
         this.actor.set_size(cornerRadius,
-                            innerBorderWidth + outerBorderWidth + cornerRadius);
+                            innerBorderWidth + cornerRadius);
         if (this._side == St.Side.LEFT)
             this.actor.set_position(Main.panel.actor.x,
-                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth - outerBorderWidth);
+                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth);
         else
             this.actor.set_position(Main.panel.actor.x + Main.panel.actor.width - cornerRadius,
-                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth - outerBorderWidth);
+                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth);
     }
 };
 
+/**
+ * HotCorner:
+ *
+ * This class manages the "hot corner" that can toggle switching to
+ * overview.
+ */
+function HotCorner() {
+    this._init();
+}
+
+HotCorner.prototype = {
+    _init : function() {
+        // We use this flag to mark the case where the user has entered the
+        // hot corner and has not left both the hot corner and a surrounding
+        // guard area (the "environs"). This avoids triggering the hot corner
+        // multiple times due to an accidental jitter.
+        this._entered = false;
+
+        this.actor = new Clutter.Group({ width: 3,
+                                         height: 3,
+                                         reactive: true });
+
+        this._corner = new Clutter.Rectangle({ width: 1,
+                                               height: 1,
+                                               opacity: 0,
+                                               reactive: true });
+
+        this.actor.add_actor(this._corner);
+
+        if (St.Widget.get_default_direction() == St.TextDirection.RTL) {
+            this._corner.set_position(this.actor.width - this._corner.width, 0);
+            this.actor.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
+        } else {
+            this._corner.set_position(0, 0);
+        }
+
+        this._activationTime = 0;
+
+        this.actor.connect('leave-event',
+                           Lang.bind(this, this._onEnvironsLeft));
+        // Clicking on the hot corner environs should result in the same bahavior
+        // as clicking on the hot corner.
+        this.actor.connect('button-release-event',
+                           Lang.bind(this, this._onCornerClicked));
+
+        // In addition to being triggered by the mouse enter event, the hot corner
+        // can be triggered by clicking on it. This is useful if the user wants to
+        // undo the effect of triggering the hot corner once in the hot corner.
+        this._corner.connect('enter-event',
+                             Lang.bind(this, this._onCornerEntered));
+        this._corner.connect('button-release-event',
+                             Lang.bind(this, this._onCornerClicked));
+        this._corner.connect('leave-event',
+                             Lang.bind(this, this._onCornerLeft));
+
+        this._corner._delegate = this._corner;
+        this._corner.handleDragOver = Lang.bind(this,
+            function(source, actor, x, y, time) {
+                 if (source == Main.xdndHandler) {
+                    if(!Main.overview.visible && !Main.overview.animationInProgress) {
+                        this.rippleAnimation();
+                        Main.overview.showTemporarily();
+                        Main.overview.beginItemDrag(actor);
+                    }
+                 }
+            });
+
+        Main.chrome.addActor(this.actor, { visibleInOverview: true, affectsStruts: false });
+    },
+
+    destroy: function() {
+        this.actor.destroy();
+    },
+
+    _addRipple : function(delay, time, startScale, startOpacity, finalScale, finalOpacity) {
+        // We draw a ripple by using a source image and animating it scaling
+        // outwards and fading away. We want the ripples to move linearly
+        // or it looks unrealistic, but if the opacity of the ripple goes
+        // linearly to zero it fades away too quickly, so we use Tweener's
+        // 'onUpdate' to give a non-linear curve to the fade-away and make
+        // it more visible in the middle section.
+
+        let [x, y] = this._corner.get_transformed_position();
+        let ripple = new St.BoxLayout({ style_class: 'ripple-box',
+                                        opacity: 255 * Math.sqrt(startOpacity),
+                                        scale_x: startScale,
+                                        scale_y: startScale,
+                                        x: x,
+                                        y: y });
+        ripple._opacity =  startOpacity;
+        Tweener.addTween(ripple, { _opacity: finalOpacity,
+                                   scale_x: finalScale,
+                                   scale_y: finalScale,
+                                   delay: delay,
+                                   time: time,
+                                   transition: 'linear',
+                                   onUpdate: function() { ripple.opacity = 255 * Math.sqrt(ripple._opacity); },
+                                   onComplete: function() { ripple.destroy(); } });
+        Main.uiGroup.add_actor(ripple);
+    },
+
+    rippleAnimation: function() {
+        // Show three concentric ripples expanding outwards; the exact
+        // parameters were found by trial and error, so don't look
+        // for them to make perfect sense mathematically
+
+        //              delay  time  scale opacity => scale opacity
+        this._addRipple(0.0,   0.83,  0.25,  1.0,    1.5,  0.0);
+        this._addRipple(0.05,  1.0,   0.0,   0.7,    1.25, 0.0);
+        this._addRipple(0.35,  1.0,   0.0,   0.3,    1,    0.0);
+    },
+
+    _onCornerEntered : function() {
+        if (!this._entered) {
+            this._entered = true;
+            if (!Main.overview.animationInProgress) {
+                this._activationTime = Date.now() / 1000;
+
+                this.rippleAnimation();
+                Main.overview.toggle();
+            }
+        }
+        return false;
+    },
+
+    _onCornerClicked : function() {
+         if (!Main.overview.animationInProgress)
+             this.maybeToggleOverviewOnClick();
+         return false;
+    },
+
+    _onCornerLeft : function(actor, event) {
+        if (event.get_related() != this.actor)
+            this._entered = false;
+        // Consume event, otherwise this will confuse onEnvironsLeft
+        return true;
+    },
+
+    _onEnvironsLeft : function(actor, event) {
+        if (event.get_related() != this._corner)
+            this._entered = false;
+        return false;
+    },
+
+    // Toggles the overview unless this is the first click on the Activities button within the HOT_CORNER_ACTIVATION_TIMEOUT time
+    // of the hot corner being triggered. This check avoids opening and closing the overview if the user both triggered the hot corner
+    // and clicked the Activities button.
+    maybeToggleOverviewOnClick: function() {
+        if (this._activationTime == 0 || Date.now() / 1000 - this._activationTime > HOT_CORNER_ACTIVATION_TIMEOUT)
+            Main.overview.toggle();
+        this._activationTime = 0;
+    }
+}
+
 
 function Panel() {
     this._init();
@@ -665,13 +814,6 @@ Panel.prototype = {
         this._centerBox = new St.BoxLayout({ name: 'panelCenter' });
         this._rightBox = new St.BoxLayout({ name: 'panelRight' });
 
-        // This will eventually be automatic, see
-        // https://bugzilla.gnome.org/show_bug.cgi?id=584662
-        if (St.Widget.get_default_direction() == St.TextDirection.RTL) {
-            this._leftBox.add_style_pseudo_class('rtl');
-            this._rightBox.add_style_pseudo_class('rtl');
-        }
-
         this._leftCorner = new PanelCorner(St.Side.LEFT);
         this._rightCorner = new PanelCorner(St.Side.RIGHT);
 
@@ -712,27 +854,6 @@ Panel.prototype = {
             sideWidth = (allocWidth - centerWidth) / 2;
 
             let childBox = new Clutter.ActorBox();
-            childBox.y1 = 0;
-            childBox.y2 = this._hotCornerEnvirons.height;
-            if (this.actor.get_direction() == St.TextDirection.RTL) {
-                childBox.x1 = allocWidth - this._hotCornerEnvirons.width;
-                childBox.x2 = allocWidth;
-            } else {
-                childBox.x1 = 0;
-                childBox.x2 = this._hotCornerEnvirons.width;
-            }
-            this._hotCornerEnvirons.allocate(childBox, flags);
-
-            childBox.y1 = 0;
-            childBox.y2 = this._hotCorner.height;
-            if (this.actor.get_direction() == St.TextDirection.RTL) {
-                childBox.x1 = allocWidth - this._hotCorner.width;
-                childBox.x2 = allocWidth;
-            } else {
-                childBox.x1 = 0;
-                childBox.x2 = this._hotCorner.width;
-            }
-            this._hotCorner.allocate(childBox, flags);
 
             childBox.y1 = 0;
             childBox.y2 = allocHeight;
@@ -800,55 +921,7 @@ Panel.prototype = {
                 corner.actor.set_style_pseudo_class(pseudoClass);
             }));
 
-        // We use this flag to mark the case where the user has entered the
-        // hot corner and has not left both the hot corner and a surrounding
-        // guard area (the "environs"). This avoids triggering the hot corner
-        // multiple times due to an accidental jitter.
-        this._hotCornerEntered = false;
-
-        this._hotCornerEnvirons = new Clutter.Rectangle({ width: 3,
-                                                          height: 3,
-                                                          opacity: 0,
-                                                          reactive: true });
-
-        this._hotCorner = new Clutter.Rectangle({ width: 1,
-                                                  height: 1,
-                                                  opacity: 0,
-                                                  reactive: true });
-
-        this._hotCornerActivationTime = 0;
-
-        this._hotCornerEnvirons.connect('leave-event',
-                                        Lang.bind(this, this._onHotCornerEnvironsLeft));
-        // Clicking on the hot corner environs should result in the same bahavior
-        // as clicking on the hot corner.
-        this._hotCornerEnvirons.connect('button-release-event',
-                                        Lang.bind(this, this._onHotCornerClicked));
-
-        // In addition to being triggered by the mouse enter event, the hot corner
-        // can be triggered by clicking on it. This is useful if the user wants to
-        // undo the effect of triggering the hot corner once in the hot corner.
-        this._hotCorner.connect('enter-event',
-                                Lang.bind(this, this._onHotCornerEntered));
-        this._hotCorner.connect('button-release-event',
-                                Lang.bind(this, this._onHotCornerClicked));
-        this._hotCorner.connect('leave-event',
-                                Lang.bind(this, this._onHotCornerLeft));
-
-        this._hotCorner._delegate = this._hotCorner;
-        this._hotCorner.handleDragOver = Lang.bind(this,
-            function(source, actor, x, y, time) {
-                 if (source == Main.xdndHandler) {
-                    if(!Main.overview.visible && !Main.overview.animationInProgress) {
-                        this.rippleAnimation();
-                        Main.overview.showTemporarily();
-                        Main.overview.beginItemDrag(actor);
-                    }
-                 }
-            });
-
-        this._boxContainer.add_actor(this._hotCornerEnvirons);
-        this._boxContainer.add_actor(this._hotCorner);
+        this._hotCorner = null;
 
         let appMenuButton = new AppMenuButton();
         this._leftBox.add(appMenuButton.actor);
@@ -894,7 +967,7 @@ Panel.prototype = {
         // to switch to.
         this.button.connect('clicked', Lang.bind(this, function(b) {
             if (!Main.overview.animationInProgress) {
-                this._maybeToggleOverviewOnClick();
+                this._hotCorner.maybeToggleOverviewOnClick();
                 return true;
             } else {
                 return false;
@@ -917,6 +990,9 @@ Panel.prototype = {
         Main.chrome.addActor(this._rightCorner.actor, { visibleInOverview: true,
                                                   affectsStruts: false,
                                                   affectsInputRegion: false });
+
+        Main.ctrlAltTabManager.addGroup(this.actor, _("Panel"), 'start-here',
+                                        { sortGroup: CtrlAltTab.SortGroup.TOP });
     },
 
     _xdndShowOverview: function (actor) {
@@ -938,6 +1014,14 @@ Panel.prototype = {
         this.button._xdndTimeOut = 0;
     },
 
+
+    // While there can be multiple hotcorners (one per monitor), the hot corner
+    // that is on top of the Activities button is special since it needs special
+    // coordination with clicking on that button
+    setHotCorner: function(corner) {
+        this._hotCorner = corner;
+    },
+
     startStatusArea: function() {
         for (let i = 0; i < STANDARD_TRAY_ICON_ORDER.length; i++) {
             let role = STANDARD_TRAY_ICON_ORDER[i];
@@ -957,12 +1041,37 @@ Panel.prototype = {
     },
 
     startupAnimation: function() {
-        this.actor.y = -this.actor.height;
+        let oldY = this.actor.y;
+        this.actor.y = oldY - this.actor.height;
         Tweener.addTween(this.actor,
-                         { y: 0,
-                           time: 0.2,
+                         { y: oldY,
+                           time: STARTUP_ANIMATION_TIME,
                            transition: 'easeOutQuad'
                          });
+
+        let oldCornerY = this._leftCorner.actor.y;
+        this._leftCorner.actor.y = oldCornerY - this.actor.height;
+        this._rightCorner.actor.y = oldCornerY - this.actor.height;
+        Tweener.addTween(this._leftCorner.actor,
+                         { y: oldCornerY,
+                           time: STARTUP_ANIMATION_TIME,
+                           transition: 'easeOutQuad'
+                         });
+        Tweener.addTween(this._rightCorner.actor,
+                         { y: oldCornerY,
+                           time: STARTUP_ANIMATION_TIME,
+                           transition: 'easeOutQuad'
+                         });
+    },
+
+    relayout: function() {
+        let primary = global.get_primary_monitor();
+
+        this.actor.set_position(primary.x, primary.y);
+        this.actor.set_size(primary.width, PANEL_HEIGHT);
+
+        this._leftCorner.relayout();
+        this._rightCorner.relayout();
     },
 
     _onTrayIconAdded: function(o, icon, role) {
@@ -1001,88 +1110,4 @@ Panel.prototype = {
             this._trayBox.remove_actor(icon);
     },
 
-    _addRipple : function(delay, time, startScale, startOpacity, finalScale, finalOpacity) {
-        // We draw a ripple by using a source image and animating it scaling
-        // outwards and fading away. We want the ripples to move linearly
-        // or it looks unrealistic, but if the opacity of the ripple goes
-        // linearly to zero it fades away too quickly, so we use Tweener's
-        // 'onUpdate' to give a non-linear curve to the fade-away and make
-        // it more visible in the middle section.
-
-        let [x, y] = this._hotCorner.get_transformed_position();
-        let ripple = new St.BoxLayout({ style_class: 'ripple-box',
-                                        opacity: 255 * Math.sqrt(startOpacity),
-                                        scale_x: startScale,
-                                        scale_y: startScale,
-                                        x: x,
-                                        y: y });
-        ripple._opacity =  startOpacity;
-        Tweener.addTween(ripple, { _opacity: finalOpacity,
-                                   scale_x: finalScale,
-                                   scale_y: finalScale,
-                                   delay: delay,
-                                   time: time,
-                                   transition: 'linear',
-                                   onUpdate: function() { ripple.opacity = 255 * Math.sqrt(ripple._opacity); },
-                                   onComplete: function() { ripple.destroy(); } });
-        Main.uiGroup.add_actor(ripple);
-    },
-
-    rippleAnimation: function() {
-        // Show three concentric ripples expanding outwards; the exact
-        // parameters were found by trial and error, so don't look
-        // for them to make perfect sense mathematically
-
-        //              delay  time  scale opacity => scale opacity
-        this._addRipple(0.0,   0.83,  0.25,  1.0,    1.5,  0.0);
-        this._addRipple(0.05,  1.0,   0.0,   0.7,    1.25, 0.0);
-        this._addRipple(0.35,  1.0,   0.0,   0.3,    1,    0.0);
-    },
-
-    _onHotCornerEntered : function() {
-        if (this._menus.grabbed)
-            return false;
-        if (!this._hotCornerEntered) {
-            this._hotCornerEntered = true;
-            if (!Main.overview.animationInProgress) {
-                this._hotCornerActivationTime = Date.now() / 1000;
-
-                this.rippleAnimation();
-                Main.overview.toggle();
-            }
-        }
-        return false;
-    },
-
-    _onHotCornerClicked : function() {
-        if (this._menus.grabbed)
-            return false;
-         if (!Main.overview.animationInProgress) {
-             this._maybeToggleOverviewOnClick();
-         }
-         return false;
-    },
-
-    _onHotCornerLeft : function(actor, event) {
-        if (event.get_related() != this._hotCornerEnvirons) {
-            this._hotCornerEntered = false;
-        }
-        return false;
-    },
-
-    _onHotCornerEnvironsLeft : function(actor, event) {
-        if (event.get_related() != this._hotCorner) {
-            this._hotCornerEntered = false;
-        }
-        return false;
-    },
-
-    // Toggles the overview unless this is the first click on the Activities button within the HOT_CORNER_ACTIVATION_TIMEOUT time
-    // of the hot corner being triggered. This check avoids opening and closing the overview if the user both triggered the hot corner
-    // and clicked the Activities button.
-    _maybeToggleOverviewOnClick: function() {
-        if (this._hotCornerActivationTime == 0 || Date.now() / 1000 - this._hotCornerActivationTime > HOT_CORNER_ACTIVATION_TIMEOUT)
-            Main.overview.toggle();
-        this._hotCornerActivationTime = 0;
-    }
 };
