@@ -1,6 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
 const Signals = imports.signals;
@@ -15,185 +16,12 @@ const Search = imports.ui.search;
 const SearchDisplay = imports.ui.searchDisplay;
 const Tweener = imports.ui.tweener;
 
-
-function SearchEntry(focusBase) {
-    this._init(focusBase);
-}
-
-SearchEntry.prototype = {
-    _init : function(focusBase) {
-        this.actor = new St.Entry({ name: 'searchEntry',
-                                    /* Translators: this is the text displayed
-                                       in the search entry when no search is
-                                       active; it should not exceed ~30
-                                       characters */
-                                    hint_text: _("Type to search..."),
-                                    track_hover: true });
-        this.entry = this.actor.clutter_text;
-
-        this._inactiveIcon = new St.Icon({ style_class: 'search-entry-icon',
-                                           icon_name: 'edit-find',
-                                           icon_type: St.IconType.SYMBOLIC });
-        this._activeIcon = new St.Icon({ style_class: 'search-entry-icon',
-                                         icon_name: 'edit-clear',
-                                         icon_type: St.IconType.SYMBOLIC });
-        this.actor.set_secondary_icon(this._inactiveIcon);
-
-        this._iconClickedId = 0;
-
-        this.actor.clutter_text.connect('text-changed',
-                                        Lang.bind(this, this._onTextChanged));
-        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
-        this.actor.connect('notify::mapped', Lang.bind(this, this._onMapped));
-
-        global.stage.connect('notify::key-focus', Lang.bind(this, this._updateCursorVisibility));
-
-        this.pane = null;
-
-        this._capturedEventId = 0;
-        this._focusBase = focusBase;
-    },
-
-    _updateCursorVisibility: function() {
-        let focus = global.stage.get_key_focus();
-        if (focus == this._focusBase || focus == this.entry)
-            this.entry.set_cursor_visible(true);
-        else
-            this.entry.set_cursor_visible(false);
-    },
-
-    _onMapped: function() {
-        if (this.actor.mapped) {
-            // Enable 'find-as-you-type'
-            this._capturedEventId = global.stage.connect('captured-event',
-                                 Lang.bind(this, this._onCapturedEvent));
-            this.entry.set_cursor_visible(true);
-            this.entry.set_selection(0, 0);
-        } else {
-            // Disable 'find-as-you-type'
-            if (this._capturedEventId > 0)
-                global.stage.disconnect(this._capturedEventId);
-            this._capturedEventId = 0;
-        }
-    },
-
-    reset: function () {
-        this.actor.sync_hover();
-
-        this.entry.text = '';
-
-        // Return focus to the viewSelector
-        global.stage.set_key_focus(this._focusBase);
-
-        this.entry.set_cursor_visible(true);
-        this.entry.set_selection(0, 0);
-    },
-
-    getText: function () {
-        return this.entry.get_text().replace(/^\s+/g, '').replace(/\s+$/g, '');
-    },
-
-    // some search term has been entered
-    isActive: function() {
-        return this.actor.get_text() != '';
-    },
-
-    // the entry does not show the hint
-    _isActivated: function() {
-        return this.entry.text == this.actor.get_text();
-    },
-
-    _onCapturedEvent: function(actor, event) {
-        let source = event.get_source();
-        let panelEvent = source && Main.panel.actor.contains(source);
-
-        switch (event.type()) {
-            case Clutter.EventType.BUTTON_PRESS:
-                // the user clicked outside after activating the entry, but
-                // with no search term entered - cancel the search
-                if (source != this.entry && this.entry.text == '') {
-                    this.reset();
-                    // allow only panel events to continue
-                    return !panelEvent;
-                }
-                return false;
-            case Clutter.EventType.KEY_PRESS:
-                // If some "special" actor grabbed the focus (run
-                // dialog, looking glass); we don't want to interfere
-                // with that
-                let focus = global.stage.get_key_focus();
-                if (focus != this._focusBase && focus != this.entry)
-                    return false;
-
-                let sym = event.get_key_symbol();
-
-                // If we have an active search, Escape cancels it - if we
-                // haven't, the key is ignored
-                if (sym == Clutter.Escape)
-                    if (this._isActivated()) {
-                        this.reset();
-                        return true;
-                    } else {
-                        return false;
-                    }
-
-                 // Ignore non-printable keys
-                 if (!Clutter.keysym_to_unicode(sym))
-                     return false;
-
-                // Search started - move the key focus to the entry and
-                // "repeat" the event
-                if (!this._isActivated()) {
-                    global.stage.set_key_focus(this.entry);
-                    this.entry.event(event, false);
-                }
-
-                return false;
-            default:
-                // Suppress all other events outside the panel while the entry
-                // is activated and no search has been entered - any click
-                // outside the entry will cancel the search
-                return (this.entry.text == '' && !panelEvent);
-        }
-    },
-
-    _onTextChanged: function() {
-        if (this.isActive()) {
-            this.actor.set_secondary_icon(this._activeIcon);
-
-            if (this._iconClickedId == 0)
-            this._iconClickedId = this.actor.connect('secondary-icon-clicked',
-                Lang.bind(this, function() {
-                    this.reset();
-                }));
-        } else {
-            if (this._iconClickedId > 0)
-                this.actor.disconnect(this._iconClickedId);
-            this._iconClickedId = 0;
-
-            this.actor.set_secondary_icon(this._inactiveIcon);
-        }
-    },
-
-    _onDestroy: function() {
-        if (this._capturedEventId > 0) {
-            global.stage.disconnect(this._capturedEventId);
-            this._capturedEventId = 0;
-        }
-
-        this._activeIcon = null;
-        this._inactiveIcon = null;
-    }
-};
-Signals.addSignalMethods(SearchEntry.prototype);
-
-
-function BaseTab(titleActor, pageActor) {
-    this._init(titleActor, pageActor);
+function BaseTab(titleActor, pageActor, name, a11yIcon) {
+    this._init(titleActor, pageActor, name, a11yIcon);
 }
 
 BaseTab.prototype = {
-    _init: function(titleActor, pageActor) {
+    _init: function(titleActor, pageActor, name, a11yIcon) {
         this.title = titleActor;
         this.page = new St.Bin({ child: pageActor,
                                  x_align: St.Align.START,
@@ -202,6 +30,14 @@ BaseTab.prototype = {
                                  y_fill: true,
                                  style_class: 'view-tab-page' });
 
+        if (this.title.can_focus) {
+            Main.ctrlAltTabManager.addGroup(this.title, name, a11yIcon);
+        } else {
+            Main.ctrlAltTabManager.addGroup(this.page, name, a11yIcon,
+                                            { proxy: this.title,
+                                              focusCallback: Lang.bind(this, this._a11yFocus) });
+        }
+
         this.visible = false;
     },
 
@@ -229,6 +65,11 @@ BaseTab.prototype = {
                          });
     },
 
+    _a11yFocus: function() {
+        this._activate();
+        this.page.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+    },
+
     _activate: function() {
         this.emit('activated');
     }
@@ -236,48 +77,68 @@ BaseTab.prototype = {
 Signals.addSignalMethods(BaseTab.prototype);
 
 
-function ViewTab(label, pageActor) {
-    this._init(label, pageActor);
+function ViewTab(label, pageActor, a11yIcon) {
+    this._init(label, pageActor, a11yIcon);
 }
 
 ViewTab.prototype = {
     __proto__: BaseTab.prototype,
 
-    _init: function(label, pageActor) {
+    _init: function(label, pageActor, a11yIcon) {
         let titleActor = new St.Button({ label: label,
                                          style_class: 'view-tab-title' });
         titleActor.connect('clicked', Lang.bind(this, this._activate));
 
-        BaseTab.prototype._init.call(this, titleActor, pageActor);
+        BaseTab.prototype._init.call(this, titleActor, pageActor, label, a11yIcon);
     }
 };
 
 
-function SearchTab(focusBase) {
-    this._init(focusBase);
+function SearchTab() {
+    this._init();
 }
 
 SearchTab.prototype = {
     __proto__: BaseTab.prototype,
 
-    _init: function(focusBase) {
-        this._searchActive = false;
+    _init: function() {
+        this.active = false;
         this._searchPending = false;
-        this._keyPressId = 0;
         this._searchTimeoutId = 0;
-        this._focusBase = focusBase;
 
         this._searchSystem = new Search.SearchSystem();
         this._openSearchSystem = new Search.OpenSearchSystem();
 
-        this._searchEntry = new SearchEntry(focusBase);
+        this._entry = new St.Entry({ name: 'searchEntry',
+                                     /* Translators: this is the text displayed
+                                        in the search entry when no search is
+                                        active; it should not exceed ~30
+                                        characters. */
+                                     hint_text: _("Type to search..."),
+                                     track_hover: true,
+                                     can_focus: true });
+        this._text = this._entry.clutter_text;
+        this._text.connect('key-press-event', Lang.bind(this, this._onKeyPress));
+
+        this._inactiveIcon = new St.Icon({ style_class: 'search-entry-icon',
+                                           icon_name: 'edit-find',
+                                           icon_type: St.IconType.SYMBOLIC });
+        this._activeIcon = new St.Icon({ style_class: 'search-entry-icon',
+                                         icon_name: 'edit-clear',
+                                         icon_type: St.IconType.SYMBOLIC });
+        this._entry.set_secondary_icon(this._inactiveIcon);
+
+        this._iconClickedId = 0;
+
         this._searchResults = new SearchDisplay.SearchResults(this._searchSystem, this._openSearchSystem);
         BaseTab.prototype._init.call(this,
-                                     this._searchEntry.actor,
-                                     this._searchResults.actor);
-        this._searchEntry.entry.connect('text-changed',
-                                        Lang.bind(this, this._onTextChanged));
-        this._searchEntry.entry.connect('activate', Lang.bind(this, function (se) {
+                                     this._entry,
+                                     this._searchResults.actor,
+                                     _("Search"),
+                                     'edit-find');
+
+        this._text.connect('text-changed', Lang.bind(this, this._onTextChanged));
+        this._text.connect('activate', Lang.bind(this, function (se) {
             if (this._searchTimeoutId > 0) {
                 Mainloop.source_remove(this._searchTimeoutId);
                 this._doSearch();
@@ -285,24 +146,51 @@ SearchTab.prototype = {
             this._searchResults.activateSelected();
             return true;
         }));
-    },
 
-    show: function() {
-        BaseTab.prototype.show.call(this);
+        this._entry.connect('secondary-icon-clicked', Lang.bind(this,
+            function() {
+                this._reset();
+            }));
+        this._entry.connect('notify::mapped', Lang.bind(this, this._onMapped));
 
-        if (this._keyPressId == 0)
-            this._keyPressId = this._searchEntry.entry.connect('key-press-event',
-                                                               Lang.bind(this, this._onKeyPress));
+        global.stage.connect('notify::key-focus', Lang.bind(this, this._updateCursorVisibility));
+
+        this._capturedEventId = 0;
     },
 
     hide: function() {
         BaseTab.prototype.hide.call(this);
 
-        if (this._keyPressId > 0) {
-            this._searchEntry.entry.disconnect(this._keyPressId);
-            this._keyPressId = 0;
+        this._reset();
+    },
+
+    _reset: function () {
+        this._text.text = '';
+
+        global.stage.set_key_focus(null);
+
+        this._text.set_cursor_visible(true);
+        this._text.set_selection(0, 0);
+    },
+
+    _updateCursorVisibility: function() {
+        let focus = global.stage.get_key_focus();
+        this._text.set_cursor_visible(focus == this._text);
+    },
+
+    _onMapped: function() {
+        if (this._entry.mapped) {
+            // Enable 'find-as-you-type'
+            this._capturedEventId = global.stage.connect('captured-event',
+                                 Lang.bind(this, this._onCapturedEvent));
+            this._text.set_cursor_visible(true);
+            this._text.set_selection(0, 0);
+        } else {
+            // Disable 'find-as-you-type'
+            if (this._capturedEventId > 0)
+                global.stage.disconnect(this._capturedEventId);
+            this._capturedEventId = 0;
         }
-        this._searchEntry.reset();
     },
 
     addSearchProvider: function(provider) {
@@ -310,19 +198,42 @@ SearchTab.prototype = {
         this._searchResults.createProviderMeta(provider);
     },
 
+    startSearch: function(event) {
+        global.stage.set_key_focus(this._text);
+        this._text.event(event, false);
+    },
+
+    // the entry does not show the hint
+    _isActivated: function() {
+        return this._text.text == this._entry.get_text();
+    },
+
     _onTextChanged: function (se, prop) {
-        let searchPreviouslyActive = this._searchActive;
-        this._searchActive = this._searchEntry.isActive();
-        this._searchPending = this._searchActive && !searchPreviouslyActive;
+        let searchPreviouslyActive = this.active;
+        this.active = this._entry.get_text() != '';
+        this._searchPending = this.active && !searchPreviouslyActive;
         if (this._searchPending) {
             this._searchResults.startingSearch();
         }
-        if (this._searchActive) {
+        if (this.active) {
+            this._entry.set_secondary_icon(this._activeIcon);
+
+            if (this._iconClickedId == 0) {
+                this._iconClickedId = this._entry.connect('secondary-icon-clicked',
+                    Lang.bind(this, function() {
+                        this.reset();
+                    }));
+            }
             this._activate();
         } else {
+            if (this._iconClickedId > 0)
+                this._entry.disconnect(this._iconClickedId);
+            this._iconClickedId = 0;
+
+            this._entry.set_secondary_icon(this._inactiveIcon);
             this.emit('search-cancelled');
         }
-        if (!this._searchActive) {
+        if (!this.active) {
             if (this._searchTimeoutId > 0) {
                 Mainloop.source_remove(this._searchTimeoutId);
                 this._searchTimeoutId = 0;
@@ -337,24 +248,43 @@ SearchTab.prototype = {
     _onKeyPress: function(entry, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.Up) {
-            if (!this._searchActive)
+            if (!this.active)
                 return true;
             this._searchResults.selectUp(false);
 
             return true;
         } else if (symbol == Clutter.Down) {
-            if (!this._searchActive)
+            if (!this.active)
                 return true;
 
             this._searchResults.selectDown(false);
             return true;
+        } else if (symbol == Clutter.Escape) {
+            if (this._isActivated()) {
+                this._reset();
+                return true;
+            }
         }
+
+        return false;
+    },
+
+    _onCapturedEvent: function(actor, event) {
+        if (event.type() == Clutter.EventType.BUTTON_PRESS) {
+            let source = event.get_source();
+            if (source != this._text && this._text.text == '') {
+                // the user clicked outside after activating the entry, but
+                // with no search term entered - cancel the search
+                this._reset();
+            }
+        }
+
         return false;
     },
 
     _doSearch: function () {
         this._searchTimeoutId = 0;
-        let text = this._searchEntry.getText();
+        let text = this._text.get_text().replace(/^\s+/g, '').replace(/\s+$/g, '');
         this._searchResults.updateSearch(text);
 
         return false;
@@ -370,8 +300,6 @@ ViewSelector.prototype = {
     _init : function() {
         this.actor = new St.BoxLayout({ name: 'viewSelector',
                                         vertical: true });
-        this.actor.connect('key-press-event',
-                           Lang.bind(this, this._onKeyPress));
 
         // The tab bar is located at the top of the view selector and
         // holds both "normal" tab labels and the search entry. The former
@@ -405,7 +333,7 @@ ViewSelector.prototype = {
         this._tabs = [];
         this._activeTab = null;
 
-        this._searchTab = new SearchTab(this.actor);
+        this._searchTab = new SearchTab();
         this._searchArea.set_child(this._searchTab.title);
         this._addTab(this._searchTab);
 
@@ -416,10 +344,22 @@ ViewSelector.prototype = {
 
         Main.overview.connect('item-drag-begin',
                               Lang.bind(this, this._switchDefaultTab));
-        Main.overview.connect('showing',
-                              Lang.bind(this, this._switchDefaultTab));
-        Main.overview.connect('hiding',
-                              Lang.bind(this, this._switchDefaultTab));
+
+        this._stageKeyPressId = 0;
+        Main.overview.connect('showing', Lang.bind(this,
+            function () {
+                this._switchDefaultTab();
+                this._stageKeyPressId = global.stage.connect('key-press-event',
+                                                             Lang.bind(this, this._onStageKeyPress));
+            }));
+        Main.overview.connect('hiding', Lang.bind(this,
+            function () {
+                this._switchDefaultTab();
+                if (this._stageKeyPressId != 0) {
+                    global.stage.disconnect(this._stageKeyPressId);
+                    this._stageKeyPressId = 0;
+                }
+            }));
 
         // Public constraints which may be used to tie actors' height or
         // vertical position to the current tab's content; as the content's
@@ -443,8 +383,8 @@ ViewSelector.prototype = {
         }));
     },
 
-    addViewTab: function(title, pageActor) {
-        let viewTab = new ViewTab(title, pageActor);
+    addViewTab: function(title, pageActor, a11yIcon) {
+        let viewTab = new ViewTab(title, pageActor, a11yIcon);
         this._tabs.push(viewTab);
         this._tabBox.add(viewTab.title);
         this._addTab(viewTab);
@@ -574,22 +514,25 @@ ViewSelector.prototype = {
             }));
     },
 
-    _onKeyPress: function(actor, event) {
+    _onStageKeyPress: function(actor, event) {
         let modifiers = Shell.get_event_state(event);
         let symbol = event.get_key_symbol();
+
         if (symbol == Clutter.Escape) {
             Main.overview.hide();
             return true;
         } else if (modifiers & Clutter.ModifierType.CONTROL_MASK) {
             if (symbol == Clutter.Page_Up) {
-                if (!this._searchActive)
+                if (!this._searchTab.active)
                     this._prevTab();
                 return true;
             } else if (symbol == Clutter.Page_Down) {
-                if (!this._searchActive)
+                if (!this._searchTab.active)
                     this._nextTab();
                 return true;
             }
+        } else if (Clutter.keysym_to_unicode(symbol)) {
+            this._searchTab.startSearch(event);
         }
         return false;
     },
