@@ -12,11 +12,10 @@ G_BEGIN_DECLS
 GType shell_app_state_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_APP_STATE (shell_app_state_get_type())
 
-/* enumerations from "shell-drawing.h" */
-GType shell_pointer_direction_get_type (void) G_GNUC_CONST;
-#define SHELL_TYPE_POINTER_DIRECTION (shell_pointer_direction_get_type())
-
 /* enumerations from "shell-global.h" */
+GType shell_cursor_get_type (void) G_GNUC_CONST;
+#define SHELL_TYPE_CURSOR (shell_cursor_get_type())
+
 GType shell_stage_input_mode_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_STAGE_INPUT_MODE (shell_stage_input_mode_get_type())
 
