@@ -2,41 +2,39 @@
 
 #include "config.h"
 
-#include "shell-global-private.h"
-#include "shell-enum-types.h"
-#include "shell-perf-log.h"
-#include "shell-window-tracker.h"
-#include "shell-marshal.h"
-#include "shell-wm.h"
-#include "st.h"
-
-#include "display.h"
-#include "util.h"
-#include <clutter/glx/clutter-glx.h>
-#include <clutter/x11/clutter-x11.h>
-#include <gdk/gdkx.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <math.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <dbus/dbus-glib.h>
-#include <gio/gio.h>
-#include <math.h>
-#include <X11/extensions/Xfixes.h>
-#include <gjs/gjs-module.h>
-#include <canberra.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
-#include "shell-jsapi-compat-private.h"
 
-#define SHELL_DBUS_SERVICE "org.gnome.Shell"
-#define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
+#include <X11/extensions/Xfixes.h>
+#include <canberra.h>
+#include <clutter/glx/clutter-glx.h>
+#include <clutter/x11/clutter-x11.h>
+#include <dbus/dbus-glib.h>
+#include <gdk/gdkx.h>
+#include <gio/gio.h>
+#include <gjs/gjs-module.h>
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+#include <libxml/xmlmemory.h>
+#include <meta/display.h>
+#include <meta/util.h>
+
+#include "shell-enum-types.h"
+#include "shell-global-private.h"
+#include "shell-jsapi-compat-private.h"
+#include "shell-marshal.h"
+#include "shell-perf-log.h"
+#include "shell-window-tracker.h"
+#include "shell-wm.h"
+#include "st.h"
 
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
 
@@ -206,7 +204,8 @@ static void
 shell_global_init (ShellGlobal *global)
 {
   const char *datadir = g_getenv ("GNOME_SHELL_DATADIR");
-  char *imagedir;
+  const char *shell_js = g_getenv("GNOME_SHELL_JS");
+  char *imagedir, **search_path;
 
   if (!datadir)
     datadir = GNOME_SHELL_DATADIR;
@@ -245,6 +244,15 @@ shell_global_init (ShellGlobal *global)
   ca_context_create (&global->sound_context);
   ca_context_change_props (global->sound_context, CA_PROP_APPLICATION_NAME, PACKAGE_NAME, CA_PROP_APPLICATION_ID, "org.gnome.Shell", NULL);
   ca_context_open (global->sound_context);
+
+  if (!shell_js)
+    shell_js = JSDIR;
+  search_path = g_strsplit (shell_js, ":", -1);
+  global->js_context = g_object_new (GJS_TYPE_CONTEXT,
+                                     "search-path", search_path,
+                                     "js-version", "1.8",
+                                     NULL);
+  g_strfreev (search_path);
 }
 
 static void
@@ -794,11 +802,10 @@ _shell_global_set_plugin (ShellGlobal *global,
   global->focus_manager = st_focus_manager_get_for_stage (CLUTTER_STAGE (stage));
 }
 
-void
-_shell_global_set_gjs_context (ShellGlobal *global,
-                               GjsContext  *context)
+GjsContext *
+_shell_global_get_gjs_context (ShellGlobal *global)
 {
-  global->js_context = context;
+  return global->js_context;
 }
 
 /**
@@ -1231,68 +1238,6 @@ shell_global_maybe_gc (ShellGlobal *global)
   gjs_context_maybe_gc (global->js_context);
 }
 
-void
-shell_global_grab_dbus_service (ShellGlobal *global)
-{
-  GError *error = NULL;
-  DBusGConnection *session;
-  DBusGProxy *bus;
-  guint32 request_name_result;
-  
-  session = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
-  
-  bus = dbus_g_proxy_new_for_name (session,
-                                   DBUS_SERVICE_DBUS,
-                                   DBUS_PATH_DBUS,
-                                   DBUS_INTERFACE_DBUS);
-  
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, SHELL_DBUS_SERVICE,
-                          G_TYPE_UINT, 0,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID)) 
-    {
-      g_print ("failed to acquire org.gnome.Shell: %s\n", error->message);
-      /* If we somehow got started again, it's not an error to be running
-       * already.  So just exit 0.
-       */
-      exit (0);  
-    }
-
-  /* Also grab org.gnome.Panel to replace any existing panel process,
-   * unless a special environment variable is passed.  The environment
-   * variable is used by the gnome-shell (no --replace) launcher in
-   * Xephyr */
-  if (!g_getenv ("GNOME_SHELL_NO_REPLACE"))
-    {
-      if (!dbus_g_proxy_call (bus, "RequestName", &error, G_TYPE_STRING,
-                              "org.gnome.Panel", G_TYPE_UINT,
-                              DBUS_NAME_FLAG_REPLACE_EXISTING | DBUS_NAME_FLAG_DO_NOT_QUEUE,
-                              G_TYPE_INVALID, G_TYPE_UINT,
-                              &request_name_result, G_TYPE_INVALID))
-        {
-          g_print ("failed to acquire org.gnome.Panel: %s\n", error->message);
-          exit (1);
-        }
-    }
-
-  /* ...and the org.gnome.Magnifier service.
-   */
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, MAGNIFIER_DBUS_SERVICE,
-                          G_TYPE_UINT, 0,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID))
-    {
-      g_print ("failed to acquire %s: %s\n", MAGNIFIER_DBUS_SERVICE, error->message);
-      /* Failing to acquire the magnifer service is not fatal.  Log the error,
-       * but keep going. */
-    }
-  g_object_unref (bus);
-}
-
 static void
 grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
 {
@@ -2049,3 +1994,57 @@ shell_get_tp_contacts (TpConnection *self,
                                        shell_global_get_tp_contacts_cb,
                                        callback, NULL, NULL);
 }
+
+/**
+ * shell_global_launch_calendar_server:
+ * @global: The #ShellGlobal.
+ *
+ * Launch the gnome-shell-calendar-server helper.
+ */
+void
+shell_global_launch_calendar_server (ShellGlobal *global)
+{
+  const gchar *bin_dir;
+  gchar *calendar_server_exe;
+  GError *error;
+  gchar *argv[2];
+  gint child_standard_input;
+
+  /* launch calendar-server */
+  bin_dir = g_getenv ("GNOME_SHELL_BINDIR");
+  if (bin_dir != NULL)
+    calendar_server_exe = g_strdup_printf ("%s/gnome-shell-calendar-server", bin_dir);
+  else
+    calendar_server_exe = g_strdup_printf (GNOME_SHELL_LIBEXECDIR "/gnome-shell-calendar-server");
+
+  argv[0] = calendar_server_exe;
+  argv[1] = NULL;
+  error = NULL;
+  if (!g_spawn_async_with_pipes (NULL, /* working_directory */
+                                 argv,
+                                 NULL, /* envp */
+                                 0, /* GSpawnFlags */
+                                 NULL, /* child_setup */
+                                 NULL, /* user_data */
+                                 NULL, /* GPid *child_pid */
+                                 &child_standard_input,
+                                 NULL, /* gint *stdout */
+                                 NULL, /* gint *stderr */
+                                 &error))
+    {
+      g_warning ("Error launching `%s': %s (%s %d)",
+                 calendar_server_exe,
+                 error->message,
+                 g_quark_to_string (error->domain),
+                 error->code);
+      g_error_free (error);
+    }
+  /* Note that gnome-shell-calendar-server exits whenever its stdin
+   * file descriptor is HUP'ed. This means that whenever the the shell
+   * process exits or is being replaced, the calendar server is also
+   * exits...and if the shell is being replaced, a new copy of the
+   * calendar server is launched...
+   */
+
+  g_free (calendar_server_exe);
+}
