@@ -2,12 +2,11 @@
 #ifndef __SHELL_GLOBAL_H__
 #define __SHELL_GLOBAL_H__
 
-#include "meta-plugin.h"
 #include <clutter/clutter.h>
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
-
+#include <meta/meta-plugin.h>
 #include <telepathy-glib/telepathy-glib.h>
 
 G_BEGIN_DECLS
@@ -59,8 +58,6 @@ gboolean shell_global_add_extension_importer (ShellGlobal *global,
                                               const char  *directory,
                                               GError     **error);
 
-void shell_global_grab_dbus_service (ShellGlobal *global);
-
 typedef enum {
   SHELL_STAGE_INPUT_MODE_NONREACTIVE,
   SHELL_STAGE_INPUT_MODE_NORMAL,
@@ -158,6 +155,8 @@ void shell_get_tp_contacts (TpConnection *self,
                             const TpContactFeature *features,
                             ShellGetTpContactCb callback);
 
+void shell_global_launch_calendar_server (ShellGlobal *global);
+
 G_END_DECLS
 
 #endif /* __SHELL_GLOBAL_H__ */
