@@ -9,6 +9,10 @@
 #include <X11/Xatom.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <meta/display.h>
+#include <meta/group.h>
+#include <meta/util.h>
+#include <meta/window.h>
 
 #define SN_API_NOT_YET_FROZEN 1
 #include <libsn/sn.h>
@@ -20,11 +24,6 @@
 #include "shell-marshal.h"
 #include "st.h"
 
-#include "display.h"
-#include "window.h"
-#include "group.h"
-#include "util.h"
-
 /* This file includes modified code from
  * desktop-data-engine/engine-dbus/hippo-application-monitor.c
  * in the functions collecting application usage data.
@@ -71,6 +70,9 @@ struct _ShellWindowTracker
 
   /* <const char *id, ShellApp *app> */
   GHashTable *running_apps;
+
+  /* <int, ShellApp *app> */
+  GHashTable *launched_pid_to_app;
 };
 
 G_DEFINE_TYPE (ShellWindowTracker, shell_window_tracker, G_TYPE_OBJECT);
@@ -379,12 +381,46 @@ get_app_from_window_group (ShellWindowTracker  *monitor,
   return result;
 }
 
+/**
+ * get_app_from_window_pid:
+ * @monitor: a #ShellWindowTracker
+ * @window: a #MetaWindow
+ *
+ * Check if the pid associated with @window corresponds to an
+ * application we launched.
+ *
+ * Return value: (transfer full): A newly-referenced #ShellApp, or %NULL
+ */
+static ShellApp *
+get_app_from_window_pid (ShellWindowTracker  *tracker,
+                         MetaWindow          *window)
+{
+  ShellApp *result;
+  int pid;
+
+  if (meta_window_is_remote (window))
+    return NULL;
+
+  pid = meta_window_get_pid (window);
+
+  if (pid == -1)
+    return NULL;
+
+  result = g_hash_table_lookup (tracker->launched_pid_to_app, GINT_TO_POINTER (pid));
+  if (result != NULL)
+    g_object_ref (result);
+
+  return result;
+}
+
 /**
  * get_app_for_window:
  *
  * Determines the application associated with a window, using
  * all available information such as the window's MetaGroup,
  * and what we know about other windows.
+ *
+ * Returns: (transfer full): a #ShellApp, or NULL if none is found
  */
 static ShellApp *
 get_app_for_window (ShellWindowTracker    *monitor,
@@ -410,6 +446,10 @@ get_app_for_window (ShellWindowTracker    *monitor,
         }
     }
 
+  result = get_app_from_window_pid (monitor, window);
+  if (result != NULL)
+    return result;
+
   /* Check if the app's WM_CLASS specifies an app */
   result = get_app_from_window_wmclass (window);
   if (result != NULL)
@@ -673,6 +713,8 @@ shell_window_tracker_init (ShellWindowTracker *self)
 
   self->running_apps = g_hash_table_new (g_str_hash, g_str_equal);
 
+  self->launched_pid_to_app = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify) g_object_unref);
+
   screen = shell_global_get_screen (shell_global_get ());
 
   g_signal_connect (G_OBJECT (screen), "startup-sequence-changed",
@@ -690,6 +732,8 @@ shell_window_tracker_finalize (GObject *object)
 
   g_hash_table_destroy (self->running_apps);
   g_hash_table_destroy (self->window_to_app);
+  g_hash_table_destroy (self->launched_pid_to_app);
+
   for (i = 0; title_patterns[i].app_id; i++)
     g_regex_unref (title_patterns[i].regex);
 
@@ -802,6 +846,40 @@ shell_window_tracker_get_running_apps (ShellWindowTracker *monitor,
   return ret;
 }
 
+static void
+on_child_exited (GPid      pid,
+                 gint      status,
+                 gpointer  unused_data)
+{
+  ShellWindowTracker *tracker;
+
+  tracker = shell_window_tracker_get_default ();
+
+  g_hash_table_remove (tracker->launched_pid_to_app, GINT_TO_POINTER((gint)pid));
+}
+
+void
+_shell_window_tracker_add_child_process_app (ShellWindowTracker *tracker,
+                                             GPid                pid,
+                                             ShellApp           *app)
+{
+  gpointer pid_ptr = GINT_TO_POINTER((int)pid);
+
+  if (g_hash_table_lookup (tracker->launched_pid_to_app,
+                           &pid_ptr))
+    return;
+
+  g_hash_table_insert (tracker->launched_pid_to_app,
+                       pid_ptr,
+                       g_object_ref (app));
+  g_child_watch_add (pid, on_child_exited, NULL);
+  /* TODO: rescan unassociated windows
+   * Unlikely in practice that the launched app gets ahead of us
+   * enough to map an X window before we get scheduled after the fork(),
+   * but adding this note for future reference.
+   */
+}
+
 static void
 set_focus_app (ShellWindowTracker  *tracker,
                ShellApp            *new_focus_app)
@@ -825,12 +903,9 @@ on_focus_window_changed (MetaDisplay        *display,
                          GParamSpec         *spec,
                          ShellWindowTracker *tracker)
 {
-  MetaScreen *screen;
   MetaWindow *new_focus_win;
   ShellApp *new_focus_app;
 
-  screen = shell_global_get_screen (shell_global_get ());
-
   new_focus_win = meta_display_get_focus_window (display);
   new_focus_app = new_focus_win ? g_hash_table_lookup (tracker->window_to_app, new_focus_win) : NULL;
 
