@@ -103,6 +103,7 @@ enum
 enum
 {
   STYLE_CHANGED,
+  POPUP_MENU,
 
   LAST_SIGNAL
 };
@@ -554,6 +555,7 @@ st_widget_get_theme_node (StWidget *widget)
       StThemeNode *parent_node = NULL;
       ClutterStage *stage = NULL;
       ClutterActor *parent;
+      char *pseudo_class, *direction_pseudo_class;
 
       parent = clutter_actor_get_parent (CLUTTER_ACTOR (widget));
       while (parent != NULL)
@@ -575,13 +577,31 @@ st_widget_get_theme_node (StWidget *widget)
       if (parent_node == NULL)
         parent_node = get_root_theme_node (CLUTTER_STAGE (stage));
 
+      /* Always append a "magic" pseudo class indicating the text
+       * direction, to allow to adapt the CSS when necessary without
+       * requiring separate style sheets.
+       */
+      if (st_widget_get_direction (widget) == ST_TEXT_DIRECTION_RTL)
+        direction_pseudo_class = "rtl";
+      else
+        direction_pseudo_class = "ltr";
+
+      if (priv->pseudo_class)
+        pseudo_class = g_strconcat(priv->pseudo_class, " ",
+                                   direction_pseudo_class, NULL);
+      else
+        pseudo_class = direction_pseudo_class;
+
       priv->theme_node = st_theme_node_new (st_theme_context_get_for_stage (stage),
                                             parent_node, priv->theme,
                                             G_OBJECT_TYPE (widget),
                                             clutter_actor_get_name (CLUTTER_ACTOR (widget)),
                                             priv->style_class,
-                                            priv->pseudo_class,
+                                            pseudo_class,
                                             priv->inline_style);
+
+      if (pseudo_class != direction_pseudo_class)
+        g_free (pseudo_class);
     }
 
   return priv->theme_node;
@@ -670,6 +690,21 @@ st_widget_key_focus_out (ClutterActor *actor)
   st_widget_remove_style_pseudo_class (widget, "focus");
 }
 
+static gboolean
+st_widget_key_press_event (ClutterActor    *actor,
+                           ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_KEY_Menu ||
+      (event->keyval == CLUTTER_KEY_F10 &&
+       (event->modifier_state & CLUTTER_SHIFT_MASK)))
+    {
+      g_signal_emit (actor, signals[POPUP_MENU], 0);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 st_widget_hide (ClutterActor *actor)
 {
@@ -744,6 +779,7 @@ st_widget_class_init (StWidgetClass *klass)
   actor_class->leave_event = st_widget_leave;
   actor_class->key_focus_in = st_widget_key_focus_in;
   actor_class->key_focus_out = st_widget_key_focus_out;
+  actor_class->key_press_event = st_widget_key_press_event;
   actor_class->hide = st_widget_hide;
 
   actor_class->get_accessible = st_widget_get_accessible;
@@ -899,6 +935,7 @@ st_widget_class_init (StWidgetClass *klass)
 
   /**
    * StWidget::style-changed:
+   * @widget: the #StWidget
    *
    * Emitted when the style information that the widget derives from the
    * theme changes
@@ -911,6 +948,22 @@ st_widget_class_init (StWidgetClass *klass)
                   NULL, NULL,
                   _st_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+
+  /**
+   * StWidget::popup-menu:
+   * @widget: the #StWidget
+   *
+   * Emitted when the user has requested a context menu (eg, via a
+   * keybinding)
+   */
+  signals[POPUP_MENU] =
+    g_signal_new ("popup-menu",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (StWidgetClass, popup_menu),
+                  NULL, NULL,
+                  _st_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 /**
@@ -1438,8 +1491,15 @@ st_widget_get_direction (StWidget *self)
 void
 st_widget_set_direction (StWidget *self, StTextDirection dir)
 {
+  StTextDirection old_direction;
+
   g_return_if_fail (ST_IS_WIDGET (self));
+
+  old_direction = st_widget_get_direction (self);
   self->priv->direction = dir;
+
+  if (old_direction != st_widget_get_direction (self))
+    st_widget_style_changed (self);
 }
 
 /**
